@@ -183,6 +183,21 @@ impl Contact {
                     required: gas_used,
                 });
             }
+
+            // check if max gas and gas used are close by seeing
+            // if we can divide max_gas by gas used, a value of one
+            // indicates that it's more than half
+            if let Some(m) = max_gas.checked_div(gas_used) {
+                if m == 1 {
+                    warn!(
+                        "Tx simulation has gas usage {} which is close to max_gas {}. \n
+                        Gas estimation is known to be inaccurate! When you submit a tx that \n
+                        requires more than the block max gas, you will not get an error message! \n
+                        Just an unexplained timeout. Watch for this.",
+                        gas_used, max_gas
+                    )
+                }
+            }
         }
 
         Ok(Fee {
